JasonWoof Got questions, comments, patches, etc.? Contact Jason Woofenden
another fix, call lt->arrange() in togglebar only
[dwm.git] / layout.c
index 9de26bc..26e9de6 100644 (file)
--- a/layout.c
+++ b/layout.c
@@ -1,4 +1,5 @@
-/* © 2004-2007 Anselm R. Garbe <garbeam at gmail dot com>
+/* © 2006-2007 Anselm R. Garbe <garbeam at gmail dot com>
+ * © 2006-2007 Sander van Dijk <a dot h dot vandijk at gmail dot com>
  * See LICENSE file for license details. */
 #include "dwm.h"
 #include <stdlib.h>
@@ -14,7 +15,7 @@ static unsigned int nmaster = NMASTER;
 
 static void
 tile(void) {
-       unsigned int i, n, nx, ny, nw, nh, mw, mh, tw, th;
+       unsigned int i, n, nx, ny, nw, nh, mw, mh, tw, th, remainder;
        Client *c;
 
        for(n = 0, c = nexttiled(clients); c; c = nexttiled(c->next))
@@ -23,6 +24,7 @@ tile(void) {
        mh = (n > nmaster) ? wah / nmaster : wah / (n > 0 ? n : 1);
        mw = (n > nmaster) ? (waw * masterw) / 1000 : waw;
        th = (n > nmaster) ? wah / (n - nmaster) : 0;
+       remainder = (n > nmaster) ? wah - th * (n - nmaster) : 0;
        tw = waw - mw;
 
        for(i = 0, c = clients; c; c = c->next)
@@ -37,18 +39,20 @@ tile(void) {
                        ny = way;
                        if(i < nmaster) {
                                ny += i * mh;
-                               nw = mw - 2 * BORDERPX;
-                               nh = mh - 2 * BORDERPX;
+                               nw = mw - 2 * c->border;
+                               nh = mh - 2 * c->border;
                        }
                        else {  /* tile window */
                                nx += mw;
-                               nw = tw - 2 * BORDERPX;
-                               if(th > 2 * BORDERPX) {
+                               nw = tw - 2 * c->border;
+                               if(th > 2 * c->border) {
                                        ny += (i - nmaster) * th;
-                                       nh = th - 2 * BORDERPX;
+                                       nh = th - 2 * c->border;
+                                       if (i == n - 1)
+                                               nh += remainder;
                                }
-                               else /* fallback if th <= 2 * BORDERPX */
-                                       nh = wah - 2 * BORDERPX;
+                               else /* fallback if th <= 2 * c->border */
+                                       nh = wah - 2 * c->border;
                        }
                        resize(c, nx, ny, nw, nh, False);
                        i++;
@@ -57,10 +61,8 @@ tile(void) {
                        c->isbanned = True;
                        XMoveWindow(dpy, c->win, c->x + 2 * sw, c->y);
                }
-       if(!sel || !isvisible(sel)) {
-               for(c = stack; c && !isvisible(c); c = c->snext);
-               focus(c);
-       }
+       if(!sel || !isvisible(sel)) 
+               focustopvisible();
        restack();
 }
 
@@ -84,10 +86,8 @@ floating(void) {
                        XMoveWindow(dpy, c->win, c->x + 2 * sw, c->y);
                }
        }
-       if(!sel || !isvisible(sel)) {
-               for(c = stack; c && !isvisible(c); c = c->snext);
-               focus(c);
-       }
+       if(!sel || !isvisible(sel))
+               focustopvisible();
        restack();
 }
 
@@ -124,7 +124,7 @@ incmasterw(const char *arg) {
                masterw = MASTERWIDTH;
        else {
                i = atoi(arg);
-               if(waw * (masterw + i) / 1000 >= waw - 2 * BORDERPX
+               if(waw * (masterw + i) / 1000 >= waw - 2 * BORDERPX 
                || waw * (masterw + i) / 1000 <= 2 * BORDERPX)
                        return;
                masterw += i;
@@ -217,6 +217,13 @@ setlayout(const char *arg) {
 }
 
 void
+togglebar(const char *arg) {
+       bpos = (bpos == BarOff) ? BARPOS : BarOff;
+       updatebarpos();
+       lt->arrange();
+}
+
+void
 togglemax(const char *arg) {
        XEvent ev;
 
@@ -237,13 +244,10 @@ togglemax(const char *arg) {
 
 void
 zoom(const char *arg) {
-       unsigned int n;
        Client *c;
 
        if(!sel || lt->arrange != tile || sel->isfloating)
                return;
-       for(n = 0, c = nexttiled(clients); c; c = nexttiled(c->next))
-               n++;
        if((c = sel) == nexttiled(clients))
                if(!(c = nexttiled(c->next)))
                        return;